import 'package:feed_plugin/feed_plugin.dart';
import 'package:feed_plugin/models/poll_option_model.dart';
import 'package:feed_plugin/models/poll_vote_model.dart';
import 'package:feed_plugin/models/user_model.dart';
import 'package:feed_plugin/repo/feed_repository.dart';
import 'package:feed_plugin/service/http_service.dart';
import 'package:feed_plugin/utilities/colors.dart';
import 'package:feed_plugin/widgets/story/story_wrapper.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'dart:async';
import 'package:intl/intl.dart';
import 'package:datetime_picker_formfield/datetime_picker_formfield.dart';
import 'package:feed_plugin/models/post_model.dart';
import 'package:feed_plugin/pages/my_feed.dart';
import 'package:feed_plugin/pages/partner_feed.dart';
import 'package:feed_plugin/pages/share_post_page.dart';
import 'package:feed_plugin/widgets/loadingButtonIndicator.dart';
import 'package:feed_plugin/widgets/post.dart';
import 'package:feed_plugin/widgets/default_profile_img.dart';
import 'package:inview_notifier_list/inview_notifier_list.dart';
import 'package:pusher_client/pusher_client.dart';
import 'dart:convert';
import 'package:empty_widget/empty_widget.dart';
import 'package:feed_plugin/utilities/global_vars.dart' as global_vars;

final lastDate = new DateTime(
  DateTime.now().year - 10,
  DateTime.now().month,
);

String selectedValue = lastDate.toString();
String? date;

class HomePage extends StatefulWidget {
  final String? token;
  final String? firstName;
  final String? lastName;
  final String? about;
  final String? profileImg;
  final String? partnerName;
  final String? partnerProfileImg;
  final partnerMedia;
  final loc;
  final List<PollOptionModel>? generalPollOptionsList;

  const HomePage({
    Key? key,
    this.token,
    this.firstName,
    this.lastName,
    this.about,
    this.profileImg,
    this.partnerName,
    this.partnerProfileImg,
    this.partnerMedia,
    this.generalPollOptionsList,
    this.loc,
  }) : super(key: key);

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage>
    with SingleTickerProviderStateMixin {
  List<PostModel> postsList = [];
  List<PostModel> postsListFilter = [];
  UserModel? currentUser;
  UserModel? currentPartner;
  bool newPostFlag = false;
  late Channel channel;
  ScrollController _scrollController = new ScrollController();
  final int likeId = 1;
  final int dislikeId = 3;
  final int saveId = 2;
  late bool flagFilter;
  int count = 0;
  late HttpService httpService;
  late FeedRepository feedRepository;
  List<PollOptionModel> pollOptionsList = [];
  List<PostModel> pollPostsList = [];
  List<PollVoteModel?> pollVotesListToQuestion = [];

  @override
  void initState() {
    httpService = HttpService();
    feedRepository = FeedRepository(httpService);
    flagFilter = false;
    feedRepository.fetchEnableLogs();
    Timer.periodic(
        Duration(minutes: 5), (Timer t) => feedRepository.fetchEnableLogs());
    initPusher();
    initCurrentUser();

    super.initState();
  }

  dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  Future<void> initPusher() async {
    channel = FeedPlugin.pusher.subscribe('post');
    channel.bind(
      'create_post',
      (onEvent) async {
        List<dynamic> newPost = json.decode(onEvent!.data!);
        List<PostModel> postsList =
            newPost.map((i) => PostModel.fromJson(i)).toList();
        List<PostModel> posts1 = postsListFilter.reversed.toList();
        feedRepository.Log(
          logName: 'PusherCreatePost',
          className: 'Pusher',
          methodName: 'pusherCreatePost',
          type: 'INFO',
          text:
              '{event: Pusher Create Post, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, post id: ${postsList[0].id}',
        );
        if (postsList[0].pollQuestion != null) {
          await feedRepository.getAllOptions().then((value) async => {
                if (mounted)
                  setState(() {
                    pollOptionsList = value;
                  })
              });
        }

        if (mounted) {
          this.setState(() {
            posts1.add(postsList[0]);
            postsListFilter = posts1.reversed.toList();
            postsList = posts1.reversed.toList();
            newPostFlag = true;
            pollPostsList =
                postsListFilter.where((p) => p.pollQuestion != null).toList();
          });
        }
      },
    );

    channel.bind(
      'remove_post',
      (onEvent) {
        int postId = int.parse(onEvent!.data!);
        feedRepository.Log(
          logName: 'PusherRemovePost',
          className: 'Pusher',
          methodName: 'pusherRemovePost',
          type: 'INFO',
          text:
              '{event: Pusher Remove Post, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, post id: $postId',
        );
        List<PostModel> posts = [];
        postsListFilter.forEach(
          (post) {
            if (post.id != postId) {
              posts.add(post);
            }
          },
        );
        if (mounted) {
          this.setState(
            () {
              postsListFilter = posts;
              postsList = posts;
            },
          );
        }
      },
    );
  }

  Future<List<PostModel>> getPosts() async {
    List<PostModel> posts = [];
    posts = await loadPosts();
    return posts;
  }

  Future<void> initCurrentUser() async {
    var partner = await getCurrentPartner();
    if (partner != null) {
      await getCurrentUser(partner.id);
    }
    getPosts();
  }

  Future<UserModel?> getCurrentUser(int? partnerId) async {
    UserModel? user;

    user = await feedRepository.authenticateUser(
      firstName: widget.firstName,
      lastName: widget.lastName,
      about: widget.about,
      profileImg: widget.profileImg,
      token: widget.token,
      socialMedia: null,
      partnerId: partnerId,
    );
    if (mounted)
      setState(() {
        currentUser = user;
      });
    return user;
  }

  Future<UserModel?> getCurrentPartner() async {
    UserModel? partner;
    bool hasPartner = widget.partnerName != null &&
        widget.partnerName != "null" &&
        widget.partnerName!.trim() != "";
    if (hasPartner) {
      partner = await feedRepository.authenticatePartner(
        firstName: widget.partnerName,
        profileImg: widget.partnerProfileImg,
        socialMedia: widget.partnerMedia,
      );
    } else {
      partner = await feedRepository.authenticatePartner(
        firstName: "phonedo",
        profileImg: global_vars.defaultPartnerImg,
        socialMedia: jsonEncode(global_vars.media),
      );
    }
    if (mounted)
      setState(() {
        currentPartner = partner;
      });
    return partner;
  }

  void startTimer(int num) {
    count = num;
    Timer.periodic(
      Duration(seconds: 1),
      (_timer) {
        if (count > 0) {
          count--;
        } else {
          _timer.cancel();
          if (mounted)
            setState(() {
              newPostFlag = false;
            });
        }
      },
    );
  }

  bool filter({required DateTime createdAt, required String selectedDate}) {
    DateTime fromDate = createdAt;
    DateTime toDate = new DateFormat("yyyy-MM-dd hh:mm:ss").parse(selectedDate);
    if (fromDate.isAfter(toDate) || toDate == fromDate) {
      return true;
    }
    return false;
  }

  bool filterByDay(
      {required DateTime createdAt, required String selectedDate}) {
    DateTime theDate =
        new DateFormat("yyyy-MM-dd hh:mm:ss").parse(selectedDate);
    setState(
      () {
        flagFilter = true;
      },
    );
    if (theDate.day == createdAt.day &&
        theDate.month == createdAt.month &&
        theDate.year == createdAt.year) {
      return true;
    }
    return false;
  }

  void getPostsByFilter() {
    List<PostModel> postsListTemp = [];

    for (var i = 0; i < postsList.length; i++) {
      if (filterByDay(
          createdAt: postsList[i].createdAt!, selectedDate: selectedValue)) {
        postsListTemp.add(postsList[i]);
      }
    }
    if (mounted)
      setState(() {
        postsListFilter = postsListTemp;
      });
  }

  List<PostModel> posts = [];

  Future<List<PostModel>> loadPosts() async {
    postsListFilter = [];
    await feedRepository.getAllOptions().then((value) async => {
          if (mounted)
            setState(() {
              pollOptionsList = value;
            })
        });
    await feedRepository.getAllVotes().then((value) async => {
          if (mounted)
            setState(() {
              global_vars.generalPollVotesList = value;
            })
        });
    await feedRepository.loadPosts().then((value) async => {
          if (mounted)
            setState(() {
              if (value.length < 1) {
                flagFilter = true;
              }
              posts = value;
              postsListFilter = posts.reversed.toList();
              postsList = posts.reversed.toList();
              pollPostsList =
                  postsListFilter.where((p) => p.pollQuestion != null).toList();
            }),
        });
    return postsListFilter;
  }

  Future refreshData() async {
    await getPosts();
    return;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: buildAppBar(),
      body: _buildBody(),
    );
  }

  PreferredSizeWidget buildAppBar() {
    return PreferredSize(
      preferredSize: Size.fromHeight(70.0),
      child: AppBar(
        automaticallyImplyLeading: false,
        backgroundColor: primaryColor,
        title: Row(
          children: [
            _buildMyFeedButton(),
            _buildTitle(),
            _buildPartnerButton(),
          ],
        ),
      ),
    );
  }

  Widget _buildPartnerButton() {
    return Container(
        margin: EdgeInsets.only(top: 12),
        child: currentUser != null && currentPartner != null
            ? GestureDetector(
                onTap: () {
                  List<PostModel> partnerPosts = postsListFilter
                      .where((p) => p.user.id == currentPartner!.id)
                      .toList();
                  Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => PartnerFeedPage(
                        loc: this.widget.loc,
                        currentUser: this.currentUser,
                        saveId: saveId,
                        likeId: likeId,
                        dislikeId: dislikeId,
                        pollOptionsList: pollOptionsList,
                        pollVotesList: global_vars.generalPollVotesList,
                        currentPartner: currentPartner,
                        partnerPosts: partnerPosts,
                      ),
                    ),
                  );
                },
                child: Container(
                  alignment: Alignment.bottomRight,
                  child: currentPartner!.profileImg == null ||
                          currentPartner!.profileImg == "" ||
                          currentPartner!.profileImg == "null"
                      ? DefaultProfileImg(
                          firstName: this.widget.partnerName,
                          heightProfileImage: 39,
                          widthProfileImage: 38,
                        )
                      : ClipOval(
                          child: Image.network(
                            currentPartner!.profileImg!,
                            width: 40,
                            height: 40,
                            fit: BoxFit.cover,
                          ),
                        ),
                ),
              )
            : Container());
  }

  Widget _buildMyFeedButton() {
    return this.currentUser != null
        ? GestureDetector(
            onTap: () async {
              List<PostModel> myPosts = postsListFilter
                  .where((p) => p.user.id == currentUser!.id)
                  .toList();
              await Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (context) => MyFeedPage(
                    currentUser: this.currentUser,
                    saveId: saveId,
                    likeId: likeId,
                    dislikeId: dislikeId,
                    postsList1: myPosts,
                    generalPollOptionsList: pollOptionsList,
                    loc: this.widget.loc,
                    withPartner: widget.partnerName != null &&
                            widget.partnerName != "null" &&
                            widget.partnerName != ""
                        ? true
                        : false,
                    pollVotesList: global_vars.generalPollVotesList,
                  ),
                ),
              ).then((value) async => {
                    await loadPosts(),
                  });
            },
            child: Padding(
              padding: EdgeInsets.only(top: 12),
              child: Container(
                alignment: Alignment.centerLeft,
                child: this.currentUser!.profileImg != null &&
                        this.currentUser!.profileImg != "" &&
                        this.currentUser!.profileImg != "null"
                    ? ClipOval(
                        child: Image.network(
                          this.currentUser!.profileImg!,
                          width: 38,
                          height: 39,
                          fit: BoxFit.cover,
                        ),
                      )
                    : DefaultProfileImg(
                        firstName: this.widget.firstName,
                        lastName: this.widget.lastName,
                        heightProfileImage: 39,
                        widthProfileImage: 38,
                      ),
              ),
            ),
          )
        : Container();
  }

  Widget _buildTitle() {
    return Expanded(
      child: Container(
        child: Text(
          this.widget.loc.translate('Feed'),
          textAlign: TextAlign.center,
          style: TextStyle(color: Colors.white),
        ),
      ),
    );
  }

  Widget _buildBody() {
    return postsListFilter.length < 1 && flagFilter
        ? Stack(
            children: [
              _buildEmptyWidget(),
              _buildSharePostButton(),
            ],
          )
        : postsListFilter.length < 1 && !flagFilter
            ? Center(child: LoadingIndicatorDark())
            : Column(
                crossAxisAlignment: CrossAxisAlignment.stretch,
                children: [
                  if (newPostFlag == true) _newPostButton(),
                  _buildDateField(),
                  Expanded(
                    child: this.currentUser != null
                        ? Stack(
                            children: [
                              _buildList(),
                              _buildSharePostButton(),
                            ],
                          )
                        : Container(),
                  ),
                ],
              );
  }

  Widget _buildEmptyWidget() {
    return GestureDetector(
      onTap: () {
        if (mounted)
          setState(
            () {
              postsListFilter = postsList;
              if (postsList.length > 0) {
                flagFilter = false;
              }
            },
          );
      },
      child: Center(
        child: EmptyWidget(
          title: this.widget.loc.translate('No Data Found'),
          subTitle: this.widget.loc.translate('Click here to see all posts'),
          packageImage: PackageImage.Image_3,
          titleTextStyle: Theme.of(context)
              .typography
              .dense
              .headline4!
              .copyWith(color: Color(0xff9da9c7)),
          subtitleTextStyle:
              Theme.of(context).typography.dense.bodyText1!.copyWith(
                    color: Color(0xffabb8d6),
                  ),
        ),
      ),
    );
  }

  Widget _newPostButton() {
    startTimer(7);
    return Column(
      children: [
        Container(
          decoration: BoxDecoration(
            border: Border(
              bottom: BorderSide(
                width: 15,
                color: appDisabledButtonColor,
              ),
            ),
          ),
        ),
        Container(
          color: Colors.white,
          height: ScreenUtil().setHeight(64),
          width: ScreenUtil().screenWidth,
          alignment: Alignment.center,
          child: SizedBox(
            height: ScreenUtil().setHeight(50),
            width: ScreenUtil().setWidth(280),
            child: Container(
              decoration: ShapeDecoration(
                color: secondColor,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      color: Colors.transparent,
                      width: 1,
                      style: BorderStyle.solid),
                  borderRadius: BorderRadius.circular(60),
                ),
              ),
              child: TextButton(
                onPressed: () {
                  startTimer(0);
                  _scrollController.animateTo(
                      _scrollController.position.minScrollExtent,
                      duration: Duration(seconds: 1),
                      curve: Curves.fastOutSlowIn);
                },
                child: Row(
                  children: [
                    Expanded(
                      flex: 2,
                      child: Container(
                        alignment: Alignment.center,
                        child: Text(
                          this.widget.loc.translate('New Post'),
                          style: TextStyle(
                            fontSize: ScreenUtil().setSp(16),
                            color: Colors.white,
                            fontFamily: 'Rubik',
                          ),
                        ),
                      ),
                    ),
                    Expanded(
                      child: Icon(
                        Icons.arrow_forward,
                        size: ScreenUtil().setWidth(25),
                        color: Colors.white,
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildDateField() {
    final format = DateFormat("dd/MM/yyyy");
    final TextEditingController controller = TextEditingController();
    final now = DateTime.now();

    final nextYear = new DateTime(
      now.year,
      now.month,
      now.day,
    );
    return Column(
      children: [
        Container(
          decoration: BoxDecoration(
            border: Border(
              bottom: BorderSide(
                width: 15,
                color: appDisabledButtonColor,
              ),
            ),
          ),
        ),
        Container(
          color: Colors.white,
          height: ScreenUtil().setHeight(50),
          width: ScreenUtil().screenWidth,
          alignment: Alignment.center,
          child: SizedBox(
            height: ScreenUtil().setHeight(40),
            width: ScreenUtil().screenWidth - ScreenUtil().setWidth(30),
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Flexible(
                  flex: 8,
                  child: Container(
                    decoration: ShapeDecoration(
                        color: appDisabledButtonColor,
                        shape: RoundedRectangleBorder(
                          side: BorderSide(
                              color: Colors.transparent,
                              width: 1,
                              style: BorderStyle.solid),
                          borderRadius: BorderRadius.circular(7),
                        )),
                    child: DateTimeField(
                      textAlign: TextAlign.center,
                      readOnly: true,
                      onChanged: (value) {
                        setState(
                          () {
                            selectedValue = value.toString();
                            date = DateFormat("dd/MM/yyyy").format(value!);
                          },
                        );
                        getPostsByFilter();
                      },
                      controller: controller,
                      format: format,
                      onShowPicker: (context, currentValue) {
                        return showDatePicker(
                            context: context,
                            firstDate: lastDate,
                            initialDate: currentValue ?? DateTime.now(),
                            lastDate: nextYear);
                      },
                      decoration: new InputDecoration(
                          prefixIcon: Icon(
                            Icons.filter_list_rounded,
                            color: appDarkBlueColor,
                          ),
                          labelText: flagFilter
                              ? date
                              : this.widget.loc.translate("filter..."),
                          labelStyle: TextStyle(
                            color: appDarkBlueColor,
                          ),
                          border:
                              OutlineInputBorder(borderSide: BorderSide.none)),
                    ),
                  ),
                ),
                Flexible(
                    flex: 1,
                    child: Container(
                      child: IconButton(
                        padding: const EdgeInsets.all(0.0),
                        onPressed: () => {
                          setState(() =>
                              {flagFilter = false, postsListFilter = postsList})
                        },
                        icon: Icon(Icons.autorenew_rounded),
                        color: appDarkBlueColor,
                      ),
                    )),
              ],
            ),
          ),
        ),
        Container(
          decoration: BoxDecoration(
            border: Border(
              bottom: BorderSide(
                width: 15,
                color: appDisabledButtonColor,
              ),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildList() {
    return RefreshIndicator(
        triggerMode: RefreshIndicatorTriggerMode.anywhere,
        onRefresh: () => getPosts(),
        child: InViewNotifierList(
          controller: _scrollController,
          scrollDirection: Axis.vertical,
          initialInViewIds: ['0'],
          isInViewPortCondition:
              (double deltaTop, double deltaBottom, double viewPortDimension) {
            return deltaTop < (0.1 * viewPortDimension) &&
                deltaBottom > (0.8 * viewPortDimension);
          },
          itemCount: this.postsListFilter.length + 1,
          builder: (BuildContext context, int index) {
            return Container(
                color: appDisabledButtonColor,
                child: LayoutBuilder(
                  builder: (BuildContext context, BoxConstraints constraints) {
                    return InViewNotifierWidget(
                      id: '$index',
                      builder:
                          (BuildContext context, bool isInView, Widget? child) {
                        if (index == 0) {
                          return _buildStoryContainer();
                        }
                        return _buildFeedPost(index, isInView);
                      },
                    );
                  },
                ));
          },
        ));
  }

  Widget _buildStoryContainer() {
    return StoryWrapper(
      loc: widget.loc,
      feedRepository: feedRepository,
      currentUser: currentUser!,
    );
  }

  Widget _buildFeedPost(int index, bool isInView) {
    List<PollOptionModel>? pollOptionsListToPost;
    PostModel post = this.postsListFilter[index - 1];
    List<String> images = postsListFilter[index - 1].attachedUrl != null &&
            postsListFilter[index - 1].attachedUrl != "null"
        ? this.postsListFilter[index - 1].attachedUrl!.split("*")
        : [];
    if (post.pollQuestion != null) {
      pollOptionsListToPost = [];
      pollOptionsListToPost = pollOptionsList
          .where((p) => p.pollQuestion!.id == post.pollQuestion!.id)
          .toList();
      pollVotesListToQuestion = [];
      pollVotesListToQuestion = global_vars.generalPollVotesList!
          .where(
              (p) => p!.pollOption!.pollQuestion!.id == post.pollQuestion!.id)
          .toList();
    }

    return FeedPost(
      key: Key("F" + post.id.toString()),
      postData: post,
      currentUser: this.currentUser,
      images: images,
      likeId: likeId,
      dislikeId: dislikeId,
      saveId: saveId,
      commentsLength: 0,
      postsList: this.postsList,
      pollOptionsList: pollOptionsListToPost,
      generalPollOptionsList: pollOptionsList,
      pollVotesList: pollVotesListToQuestion,
      loc: this.widget.loc,
      play: isInView,
    );
  }

  Widget _buildSharePostButton() {
    return currentUser != null && currentUser!.enabled == true
        ? Align(
            alignment: Alignment(0.8, 0.65),
            child: Container(
              decoration: BoxDecoration(
                shape: BoxShape.circle,
                boxShadow: [
                  BoxShadow(
                    color: Colors.grey,
                    blurRadius: 5.0,
                    spreadRadius: 1.0,
                  )
                ],
              ),
              child: ClipOval(
                child: Material(
                  color: secondColor,
                  child: InkWell(
                    child: SizedBox(
                      width: 70,
                      height: 70,
                      child: Icon(Icons.add, size: 50.0, color: Colors.white),
                    ),
                    onTap: () {
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                            builder: (context) => SharePostPage(
                                currentUser: this.currentUser,
                                loc: this.widget.loc)),
                      );
                    },
                  ),
                ),
              ),
            ),
          )
        : Container();
  }
}
