import 'dart:io';
import 'dart:typed_data';
import 'package:feed_plugin/models/hashtag.dart';
import 'package:feed_plugin/models/poll_question_model.dart';
import 'package:feed_plugin/models/user_model.dart';
import 'package:feed_plugin/models/media_source.dart';
import 'package:feed_plugin/pages/source_page.dart';
import 'package:feed_plugin/repo/feed_repository.dart';
import 'package:feed_plugin/service/http_service.dart';
import 'package:feed_plugin/utilities/colors.dart';
import 'package:feed_plugin/utilities/hashTag.dart';
import 'package:feed_plugin/widgets/default_profile_img.dart';
import 'package:feed_plugin/widgets/video_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:giphy_picker/giphy_picker.dart';
import 'package:cloudinary_public/cloudinary_public.dart';
import 'package:image_picker/image_picker.dart';
import 'package:status_alert/status_alert.dart';
import 'dart:async';

import 'package:video_compress/video_compress.dart';

class SharePostPage extends StatefulWidget {
  SharePostPage({
    Key? key,
    required this.currentUser,
    required this.loc,
  }) : super(key: key);

  final UserModel? currentUser;
  final loc;
  @override
  _SharePostPageState createState() => _SharePostPageState();
}

class _SharePostPageState extends State<SharePostPage> {
  FocusNode? focusNodeTitle;

  FocusNode? focusNodeDescription;

  late HttpService httpService;

  late FeedRepository feedRepository;

  String url = "https://api.cloudinary.com/v1_1/feed-phonedo";

  String? pluginTitle;

  GiphyGif? _gif;
  final TextEditingController descriptionController = HashTag(
    {
      r"\B#\w\w+": TextStyle(color: Colors.blue),
    },
  );
  final titleController = TextEditingController();

  final _formKey = GlobalKey<FormState>();

  final cloudinary = CloudinaryPublic('feed-phonedo', 'edwhfpti', cache: false);

  var fileUrl;

  File? imageEdit;

  List<File> images = [];

  String jsonUrl = "";

  String? myType;

  bool isPoll = false;

  bool isPollForm = false;

  PollQuestionModel? pollQuestion;

  File? fileMedia;

  MediaSource? source;

  var validColor = false;

  late var contextShare;

  bool isOption3 = false;

  bool isOption4 = false;

  bool isValid = false;

  var questionController = TextEditingController();

  var option1Controller = TextEditingController();

  var option2Controller = TextEditingController();

  var option3Controller = TextEditingController();

  var option4Controller = TextEditingController();

  FocusNode? questionFocusNode;

  FocusNode? option1FocusNode;

  FocusNode? option2FocusNode;

  FocusNode? option3FocusNode;

  FocusNode? option4FocusNode;

  bool? secondsFlag;
  Uint8List? thumbnailBytes;
  int? videoSize;
  MediaInfo? compressVideoInfo;

  @override
  void initState() {
    httpService = HttpService();
    feedRepository = FeedRepository(httpService);
    focusNodeTitle = FocusNode();
    focusNodeDescription = FocusNode();
    questionFocusNode = FocusNode();
    option1FocusNode = FocusNode();
    option2FocusNode = FocusNode();
    option3FocusNode = FocusNode();
    option4FocusNode = FocusNode();
    super.initState();
  }

  dynamic checkIsHebrew(String text) {
    var enLen;
    var heLen;
    if (text != "" && text.contains(RegExp(r'[א-ת]'))) {
      heLen = 1;
    } else if (text != "") {
      enLen = 1;
    }
    if (heLen == 1) return true;

    return false;
  }

  Future<void> loadAssets() async {
    final ImagePicker _picker = ImagePicker();
    final List<XFile> imgs = await _picker.pickMultiImage(imageQuality: 50);
    List<File> resultList = [];
    if (images.length > 0) {
      resultList = images;
    }
    int n = imgs.length + resultList.length > 4
        ? 4 - resultList.length
        : imgs.length;

    for (var i = 0; i < n; i++) {
      File file = File(imgs[i].path);
      resultList.add(file);
    }

    setState(() {
      images = resultList;
      images.length == 0
          ? myType = null
          : images.length == 4
              ? myType = "0"
              : myType = "Myimg";
    });

    if (imgs.length > 4) {
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          content:
              Text(widget.loc.translate("Can't upload more than 4 photos!")),
        ),
      );
    }
  }

  // Future<void> getimageditor() async {
  //   focusNodeTitle.unfocus();
  //   focusNodeDescription.unfocus();
  //   Navigator.push(
  //     context,
  //     MaterialPageRoute(
  //       builder: (context) {
  //         return ImageEditorPro(
  //           appBarColor:primaryColor,
  //           bottomBarColor:primaryColor,
  //         );
  //       },
  //     ),
  //   ).then((geteditimage) {
  //     if (geteditimage != null) {
  //       setState(() {
  //         imageEdit = geteditimage;
  //       });
  //       myType = "Myimgeditor";
  //     }
  //   }).catchError((er) {});
  // }

  Future capture(MediaSource source) async {
    focusNodeTitle!.unfocus();
    focusNodeDescription!.unfocus();
    setState(() {
      this.source = source;
      this.fileMedia = null;
      this.compressVideoInfo = null;
    });

    final result = await Navigator.of(context).push(
      MaterialPageRoute(
        builder: (context) => SourcePage(loc: this.widget.loc),
        settings: RouteSettings(
          arguments: source,
        ),
      ),
    );

    if (result == null) {
      return;
    } else {
      setState(() {
        validColor = false;
        fileMedia = result;
        myType = "Myvideo";
      });
      generateThumbnail(fileMedia!);
      getVideoSize(fileMedia!);
      compressVideo(fileMedia);
    }
  }

  Future generateThumbnail(File file) async {
    final thumbnailBytes = await VideoCompress.getByteThumbnail(file.path);
    setState(() {
      this.thumbnailBytes = thumbnailBytes;
    });
  }

  Future getVideoSize(File file) async {
    final size = await file.length();
    setState(() {
      videoSize = size;
    });
  }

  Future compressVideo(File? file) async {
    final info = await VideoCompressApi.compressVideo(fileMedia!);
    setState(() => compressVideoInfo = info);
  }

  Future uploadFile(file, type, currentUser) async {
    CloudinaryResponse? response;
    var s = "";
    if (type == "Photo") {
      if (images.length > 0) {
        String url1;
        List<String> check;
        images.forEach((img) async {
          cloudinary
              .uploadFile(CloudinaryFile.fromFile(
                img.path,
                resourceType: CloudinaryResourceType.Image,
                // moderation: "webpurify"
              ))
              .then((res) => {
                    url1 = res.url,
                    s = url1.toString().replaceAll('http:', 'https:'),
                    if (this.jsonUrl == "")
                      {
                        this.jsonUrl += s,
                      }
                    else
                      {
                        this.jsonUrl += "*" + s,
                      },
                    check = this.jsonUrl.split("*"),
                    if (check.length == images.length)
                      {
                        setState(() {
                          fileUrl = this.jsonUrl;
                        }),
                        addPost2(currentUser),
                      }
                  });
        });
      } else {
        validColor == true ? _onLoading(true) : _onLoading(false);
        response = await cloudinary.uploadFile(
          CloudinaryFile.fromFile(file.path,
              resourceType: CloudinaryResourceType.Image),
        );
        var url1 = response.url;
        s = url1.replaceAll('http:', 'https:');
      }
    } else if (type == "Video" && compressVideoInfo != null) {
      validColor == true ? _onLoading(true) : _onLoading(false);
      response = await cloudinary.uploadFile(
        CloudinaryFile.fromFile(file.path,
            resourceType: CloudinaryResourceType.Video),
      );
      var urlV = response.url;
      s = urlV.replaceAll('http:', 'https:');
    }
    if (response != null) {
      setState(() {
        fileUrl = s;
      });
      addPost2(currentUser);
    }
  }

  _onLoading(bool flag) {
    if (flag) {
      showDialog(
          context: context,
          barrierDismissible: false,
          builder: (BuildContext context) {
            contextShare = context;
            return Center(child: CircularProgressIndicator());
          });
    } else {
      Navigator.of(contextShare).pop();
    }
  }

  addHashtag(postId) async {
    List<HashtagModel> tags = [];
    String text = titleController.text + " " + descriptionController.text;
    RegExp exp = new RegExp(r"\B#\w\w+");
    if (exp.allMatches(text).length > 0) {
      await feedRepository.getAllHashtags().then(
            (value) async => {
              tags = value,
              exp.allMatches(text).forEach((match) async {
                var hashtag = match.group(0);
                HashtagModel? thisTag;
                for (var i = 0; i < tags.length; i++) {
                  if (tags[i].tagName == hashtag) {
                    thisTag = tags[i];
                    break;
                  }
                }
                if (thisTag != null) {
                  await feedRepository.addhashtagPosts(thisTag.id, postId);
                } else {
                  await feedRepository.addNewhashtag(hashtag).then(
                      (value) async => {
                            await feedRepository.addhashtagPosts(
                                value!.id, postId)
                          });
                }
              })
            },
          );
    }
  }

  addPost(UserModel? currentUser) async {
    _onLoading(true);
    StatusAlert.show(
      context,
      duration: Duration(seconds: 2),
      title: this
          .widget
          .loc
          .translate('The post will be published in these moments'),
      configuration: IconConfiguration(
        icon: Icons.mark_email_read_outlined,
        size: 70,
      ),
    );
    if (images.length > 0) {
      pluginTitle = "Photo";
      await uploadFile(images, pluginTitle, currentUser);
    } else if (fileMedia != null) {
      pluginTitle = "Video";
      await uploadFile(
          compressVideoInfo == null ? null : compressVideoInfo!.file,
          pluginTitle,
          currentUser);
    } else if (_gif != null) {
      pluginTitle = "Photo";
      fileUrl = _gif!.images.original!.url;
      addPost2(currentUser);
    } else if (imageEdit != null) {
      pluginTitle = "Photo";
      await uploadFile(imageEdit, pluginTitle, currentUser);
    } else {
      addPost2(currentUser);
    }
  }

  validationPost(currentUser) {
    showDialog(
        context: context,
        barrierDismissible: false,
        builder: (BuildContext context1) {
          return AlertDialog(
            content: Text(this.widget.loc.translate(
                "The feed is for entertainment purposes and you may not post support issues,offensive content or content against the company.If you do this the post will be removed and you will be blocked from the system permanently.")),
            actions: [
              TextButton(
                onPressed: () {
                  Navigator.of(context).pop();
                },
                child: Text(this.widget.loc.translate("Cancel")),
              ),
              TextButton(
                onPressed: () {
                  addPost(currentUser);
                },
                child: Text(this.widget.loc.translate("I agree")),
              ),
            ],
          );
        });
  }

  addPost2(currentUser) async {
    if (isValid) {
      pollQuestion =
          await feedRepository.createPollQuestion(questionController.text);
      await feedRepository.createPollOption(
          pollQuestion!.id, option1Controller.text);
      await feedRepository.createPollOption(
          pollQuestion!.id, option2Controller.text);
      if (isOption3) {
        await feedRepository.createPollOption(
            pollQuestion!.id, option3Controller.text);
      }
      if (isOption4) {
        await feedRepository.createPollOption(
            pollQuestion!.id, option4Controller.text);
      }
    }

    if ((fileUrl != null &&
            (pluginTitle == 'Photo' || pluginTitle == 'Video')) ||
        (pluginTitle != 'Photo' && pluginTitle != 'Video')) {
      feedRepository
          .addPost(
              user: currentUser,
              title: titleController.text,
              description: descriptionController.text,
              type: pluginTitle,
              attachedUrl: fileUrl,
              pollQuestionId: isValid ? pollQuestion!.id : null)
          .then(
            (value) async => {
              if (value != null) await addHashtag(value.id),
              StatusAlert.hide(),
              value == null
                  ? {
                      _onLoading(false),
                      StatusAlert.show(
                        context,
                        duration: Duration(seconds: 1),
                        title: this.widget.loc.translate('No post received'),
                        configuration: IconConfiguration(
                          icon: Icons.mark_email_unread_outlined,
                          size: 70,
                        ),
                      )
                    }
                  : {
                      _onLoading(false),
                      Navigator.of(context).pop(),
                      Navigator.of(context).pop(),
                      if (pluginTitle == "Video") Navigator.of(context).pop(),
                    },
            },
          );
    }
  }

  @override
  Widget build(BuildContext context) {
    var width = MediaQuery.of(context).size.width;
    var height = MediaQuery.of(context).size.height;
    return Directionality(
      textDirection: TextDirection.ltr,
      child: ScreenUtilInit(
        designSize: Size(width, height),
        builder: (context, widget) =>
            isPollForm ? _buildPollScaffold() : _buildSharePostScaffold(),
      ),
    );
  }

  //-------------Poll Scaffold-------------------------------
  _buildPollScaffold() {
    return Scaffold(
      resizeToAvoidBottomInset: true,
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(70.0),
        child: AppBar(
          actions: <Widget>[
            _buildDoneButton(),
          ],
          leading: _buildPollBackButton(),
          title: _buildPollTitle(),
          backgroundColor: primaryColor,
        ),
      ),
      body: _buildPollBody(),
    );
  }

  _buildDoneButton() {
    return Align(
      alignment: Alignment.bottomRight,
      child: TextButton(
        onPressed: isValid
            ? () {
                setState(() {
                  isPoll = true;
                  isPollForm = false;
                });
              }
            : null,
        child: Text(
          widget.loc.translate('Done'),
          style: TextStyle(
            color: isValid ? Colors.white : Colors.grey,
          ),
        ),
      ),
    );
  }

  _buildPollBackButton() {
    return Padding(
      padding: EdgeInsets.only(
        top: ScreenUtil().setWidth(9),
      ),
      child: IconButton(
        onPressed: () {
          setState(() {
            isPollForm = false;
          });
        },
        icon: Icon(Icons.arrow_back, color: Colors.white),
      ),
    );
  }

  _buildPollTitle() {
    return Padding(
      padding: EdgeInsets.only(
        top: ScreenUtil().setWidth(9),
      ),
      child: Text(widget.loc.translate("Create a poll")),
    );
  }

  _buildPollBody() {
    return Padding(
      padding: EdgeInsets.all(
        ScreenUtil().setHeight(20),
      ),
      child: ListView(
        children: [
          _buildField(
            labelText: widget.loc.translate('Your question *'),
            hintText: widget.loc.translate('e.g., How do you commute to work?'),
            maxLength: 140,
            isQuestion: true,
            controller: questionController,
            focusNode: questionFocusNode,
          ),
          _buildField(
            labelText: widget.loc.translate('Option 1 *'),
            hintText: widget.loc.translate('e.g., Public Transport'),
            maxLength: 50,
            isQuestion: false,
            controller: option1Controller,
            focusNode: option1FocusNode,
          ),
          _buildField(
            labelText: widget.loc.translate('Option 2 *'),
            hintText: widget.loc.translate('e.g., Walking'),
            maxLength: 50,
            isQuestion: false,
            controller: option2Controller,
            focusNode: option2FocusNode,
          ),
          isOption3
              ? Column(
                  children: [
                    _buildRemoveButton(false),
                    _buildField(
                      labelText: widget.loc.translate('Option 3'),
                      hintText: widget.loc.translate('e.g., Carpool'),
                      maxLength: 50,
                      isQuestion: false,
                      controller: option3Controller,
                      focusNode: option3FocusNode,
                    )
                  ],
                )
              : Text(""),
          isOption4
              ? Column(
                  children: [
                    _buildRemoveButton(true),
                    _buildField(
                      labelText: widget.loc.translate('Option 4'),
                      hintText: widget.loc.translate('e.g., Drive myself'),
                      maxLength: 50,
                      isQuestion: false,
                      controller: option4Controller,
                      focusNode: option4FocusNode,
                    )
                  ],
                )
              : Text(""),
          !isOption4 ? _buildAddOptionButton() : Text(""),
        ],
      ),
    );
  }

  _buildField({
    required String? labelText,
    required String? hintText,
    required int maxLength,
    required bool isQuestion,
    required TextEditingController controller,
    required FocusNode? focusNode,
  }) {
    return TextField(
      focusNode: focusNode,
      onChanged: (text) {
        if (questionController.text.isEmpty == false &&
            option1Controller.text.isEmpty == false &&
            option2Controller.text.isEmpty == false &&
            isValid == false) {
          setState(
            () {
              isValid = true;
            },
          );
        } else if (questionController.text.isEmpty ||
            option1Controller.text.isEmpty ||
            option2Controller.text.isEmpty) {
          if (isValid == true) {
            setState(
              () {
                isValid = false;
              },
            );
          }
        }
      },
      controller: controller,
      maxLength: maxLength,
      maxLines: isQuestion == true ? null : 1,
      decoration: new InputDecoration(
        border: new OutlineInputBorder(
          borderSide: new BorderSide(
            color: Colors.teal,
          ),
        ),
        hintText: hintText,
        labelText: labelText,
      ),
    );
  }

  _buildRemoveButton(bool flag4) {
    return Align(
      alignment: Alignment.bottomRight,
      child: TextButton(
        onPressed: () {
          setState(
            () {
              if (isOption3 == true) {
                if (isOption4 == false) {
                  isOption3 = false;
                  option3Controller.text = "";
                } else {
                  if (flag4 == false) {
                    isOption4 = false;
                    option3Controller.text = option4Controller.text;
                    option4Controller.text = "";
                  } else {
                    isOption4 = false;
                    option4Controller.text = "";
                  }
                }
              }
            },
          );
        },
        child: Text(
          widget.loc.translate("Remove"),
          style: TextStyle(
            color: Colors.blue,
          ),
        ),
      ),
    );
  }

  _buildAddOptionButton() {
    return Align(
      alignment: Alignment.bottomLeft,
      child: TextButton(
        style: TextButton.styleFrom(
          shape: RoundedRectangleBorder(
            borderRadius: new BorderRadius.circular(25),
            side: BorderSide(color: Colors.blue),
          ),
        ),
        onPressed: () {
          isOption3 == false
              ? option3FocusNode!.requestFocus()
              : option4FocusNode!.requestFocus();

          setState(
            () {
              isOption3 == false ? isOption3 = true : isOption4 = true;
            },
          );
        },
        child: Text(
          '+ Add Option',
          style: TextStyle(
            color: Colors.blue,
          ),
        ),
      ),
    );
  }

  //-------------Share Post Scaffold-------------------------------
  _buildSharePostScaffold() {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(70.0),
        child: AppBar(
          backgroundColor: primaryColor,
          leading: _buildShareCloseButton(),
          title: _buildShareTitle(),
        ),
      ),
      body: _buildShareBody(),
    );
  }

  _buildShareCloseButton() {
    return Padding(
      padding: EdgeInsets.only(
        top: ScreenUtil().setWidth(9),
      ),
      child: IconButton(
        onPressed: () {
          Navigator.pop(context);
        },
        icon: Icon(Icons.close, color: Colors.white),
      ),
    );
  }

  _buildShareTitle() {
    return Container(
      alignment: Alignment(-0.3, 0),
      child: Padding(
        padding: EdgeInsets.only(top: ScreenUtil().setWidth(9)),
        child: Text(
          this.widget.loc.translate("Share post"),
          textAlign: TextAlign.center,
          style: TextStyle(
            color: Colors.white,
          ),
        ),
      ),
    );
  }

  _buildShareBody() {
    return Directionality(
      textDirection: TextDirection.ltr,
      child: Stack(children: <Widget>[
        ListView.separated(
          itemCount: 1,
          itemBuilder: (BuildContext context, int index) {
            return _buildShareContent(this.widget.currentUser);
          },
          separatorBuilder: (BuildContext context, int index) =>
              const Divider(),
        ),
        _buildShareButtomBar()
      ]),
    );
  }

  _buildShareContent(currentUser) {
    return Form(
      key: _formKey,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          _buildProfile(currentUser),
          _buildTitle(),
          _buildDescription(),
          isPoll == true ? _buildPollWidget() : Container(),
          images.length > 0
              ? _buildGridView()
              : _gif != null
                  ? _buildGifContainer()
                  : imageEdit != null
                      ? _buildImageEditContainer()
                      : fileMedia != null
                          ? _buildVideoContainer()
                          : Container(
                              height: ScreenUtil().setHeight(240),
                            ),
        ],
      ),
    );
  }

  _buildProfile(currentUser) {
    return Container(
        child: Padding(
            padding: EdgeInsets.only(
              left: ScreenUtil().setHeight(17),
              top: ScreenUtil().setHeight(20),
              bottom: ScreenUtil().setHeight(30),
            ),
            child: Row(children: [
              Column(
                children: [
                  Container(
                      margin: EdgeInsets.only(
                        right: ScreenUtil().setWidth(10),
                      ),
                      child: currentUser.profileImg == null ||
                              currentUser.profileImg == "" ||
                              currentUser.profileImg == "null"
                          ? DefaultProfileImg(
                              firstName: currentUser.firstName,
                              lastName: currentUser.lastName,
                              heightProfileImage: 39,
                              widthProfileImage: 38,
                            )
                          : ClipOval(
                              child: Image.network(
                                currentUser.profileImg,
                                width: ScreenUtil().setWidth(38),
                                height: ScreenUtil().setWidth(39),
                                fit: BoxFit.cover,
                              ),
                            )),
                ],
              ),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Container(
                    width:
                        ScreenUtil().screenWidth - ScreenUtil().setWidth(163),
                    child: Text(
                      currentUser.lastName != null &&
                              currentUser.lastName != "null"
                          ? currentUser.firstName + " " + currentUser.lastName
                          : currentUser.firstName,
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      style: TextStyle(
                        fontWeight: FontWeight.w500,
                        color: Colors.black,
                        fontSize: ScreenUtil().setSp(15),
                        fontFamily: 'Rubik',
                      ),
                    ),
                  ),
                  Container(
                    width:
                        ScreenUtil().screenWidth - ScreenUtil().setWidth(165),
                    child: Text(
                      currentUser.about != null && currentUser.about != "null"
                          ? currentUser.about
                          : "",
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      style: TextStyle(
                        fontWeight: FontWeight.normal,
                        color: Colors.black.withOpacity(0.4),
                        fontSize: ScreenUtil().setSp(13),
                        fontFamily: "Rubik",
                      ),
                    ),
                  ),
                ],
              ),
            ])));
  }

  _buildTitle() {
    return Container(
      padding: EdgeInsets.only(
        left: ScreenUtil().setHeight(17),
        bottom: ScreenUtil().setHeight(17),
      ),
      child: TextFormField(
        textDirection: checkIsHebrew(titleController.text)
            ? TextDirection.rtl
            : TextDirection.ltr,
        focusNode: focusNodeTitle,
        onChanged: (text) {
          if (titleController.text.isEmpty ||
              descriptionController.text.isEmpty) {
            setState(() {
              validColor = false;
            });
          } else {
            setState(() {
              validColor = true;
            });
          }
        },
        style: TextStyle(fontSize: 15),
        minLines: 1,
        maxLines: 1,
        decoration: InputDecoration.collapsed(
            hintText: this.widget.loc.translate("Please type title")),
        controller: titleController,
      ),
    );
  }

  _buildDescription() {
    return Container(
      padding: EdgeInsets.only(
        left: ScreenUtil().setHeight(17),
      ),
      constraints: BoxConstraints(maxHeight: ScreenUtil().setHeight(800)),
      child: _buildHashtagField(),
    );
  }

//----poll{
  _buildPollWidget() {
    return isPoll == true
        ? Padding(
            padding: EdgeInsets.all(
              ScreenUtil().setHeight(20),
            ),
            child: Container(
              width: ScreenUtil().screenWidth - ScreenUtil().setHeight(20),
              decoration: BoxDecoration(
                border: Border.all(color: Colors.grey),
                borderRadius: new BorderRadius.circular(5),
              ),
              child: Padding(
                padding: EdgeInsets.all(
                  ScreenUtil().setHeight(8),
                ),
                child: Column(
                  children: [
                    Row(
                      children: [
                        _buildCloseEditButton(
                          isEdit: false,
                        ),
                        _buildCloseEditButton(
                          isEdit: true,
                        ),
                      ],
                    ),
                    Text(
                      this.questionController.text,
                      style: TextStyle(fontWeight: FontWeight.bold),
                    ),
                    _buildOptionButton(text: this.option1Controller.text),
                    _buildOptionButton(text: this.option2Controller.text),
                    Visibility(
                      child:
                          _buildOptionButton(text: this.option3Controller.text),
                      visible: this.option3Controller.text != "",
                    ),
                    Visibility(
                      child:
                          _buildOptionButton(text: this.option4Controller.text),
                      visible: this.option4Controller.text != "",
                    ),
                    Row(
                      children: [
                        TextButton(
                          child: Text('0 vote'),
                          onPressed: () {
                            print("vote");
                          },
                        ),
                        TextButton(
                          child:
                              Text(this.widget.loc.translate('View results')),
                          onPressed: () {
                            print("view result");
                          },
                        )
                      ],
                    ),
                  ],
                ),
              ),
            ),
          )
        : Text("");
  }

  _buildOptionButton({required String text}) {
    return SizedBox(
      width: double.infinity,
      child: TextButton(
        style: TextButton.styleFrom(
          shape: RoundedRectangleBorder(
            borderRadius: new BorderRadius.circular(25),
            side: BorderSide(color: Colors.blue),
          ),
        ),
        onPressed: () {
          print('print');
        },
        child: Text(
          text,
          style: TextStyle(
            color: Colors.blue,
          ),
        ),
      ),
    );
  }

  _buildCloseEditButton({required bool isEdit}) {
    return IconButton(
      onPressed: isEdit
          ? () {
              WidgetsBinding.instance.addPostFrameCallback((_) {
                setState(() {
                  isPollForm = true;
                });
              });
            }
          : () {
              setState(() {
                this.isPoll = false;
                this.isValid = false;
                questionController.text = "";
                option1Controller.text = "";
                option2Controller.text = "";
                option3Controller.text = "";
                option4Controller.text = "";
              });
            },
      icon: isEdit
          ? Icon(
              Icons.edit,
            )
          : Icon(
              Icons.close,
            ),
    );
  }

//---poll}

  _deleteIcon(param, index) {
    return GestureDetector(
      onTap: () {
        setState(() {
          if (param == fileMedia) {
            fileMedia = null;
            myType = null;
            secondsFlag = null;
          } else if (param == _gif) {
            _gif = null;
            myType = null;
          } else if (param == imageEdit) {
            imageEdit = null;
            myType = null;
          } else {
            images.remove(images[index]);
            images.length < 4 && images.length != 0
                ? myType = "Myimg"
                : myType = null;
          }
        });
      },
      child: Padding(
        padding: const EdgeInsets.all(3.0),
        child: CircleAvatar(
          radius: 12.0,
          backgroundColor: Colors.white,
          child: Icon(Icons.close, color: Colors.grey),
        ),
      ),
    );
  }

  _buildGridView() {
    return Container(
        height: ScreenUtil().screenHeight / 2,
        child: images != null
            ? GridView.count(
                shrinkWrap: true,
                crossAxisCount: 2,
                children: List.generate(
                  images.length,
                  (index) {
                    return Stack(
                      children: [
                        SizedBox(
                          width: 450,
                          height: 440,
                          child: Image.file(images[index], fit: BoxFit.fill),
                        ),
                        _deleteIcon(images, index)
                      ],
                    );
                  },
                ),
              )
            : Container());
  }

  _buildGifContainer() {
    return Container(
        height: ScreenUtil().setHeight(240),
        child: Stack(
          children: [
            Center(child: GiphyImage.original(gif: _gif!)),
            _gif != null ? _deleteIcon(_gif, 0) : null
          ],
        ));
  }

  _buildImageEditContainer() {
    return Stack(
      children: [
        Center(
          child: Container(
              child: Image.file(imageEdit!),
              height: ScreenUtil().setHeight(240)),
        ),
        _deleteIcon(imageEdit, 0)
      ],
    );
  }

  _buildVideoContainer() {
    return Column(
      children: [
        Container(
            height: ScreenUtil().setHeight(240),
            child: Stack(
              children: [
                Center(
                  child: Container(
                    // height: ScreenUtil().setHeight(240),
                    child: compressVideoInfo == null
                        ? CircularProgressIndicator()
                        : (source == MediaSource.video && fileMedia != null
                            ? VideoWidget(
                                loc: this.widget.loc,
                                file: fileMedia!,
                                flag: (val) {
                                  setState(() {
                                    secondsFlag = val;
                                  });
                                },
                                seconds: 120,
                              )
                            : Container()),
                  ),
                ),
                _deleteIcon(fileMedia, 0)
              ],
            )),
        if ((videoSize != null && validColor == false) &&
            compressVideoInfo == null)
          Container(
              child: Text(
            this.widget.loc.translate("Thanks for sharing!") +
                "\n\n" +
                this.widget.loc.translate(" You uploaded a heavy file,") +
                "\n\n" +
                this.widget.loc.translate(" Sorry for the delay!") +
                "\n\n" +
                this.widget.loc.translate("Please wait for Loading..."),
            style: TextStyle(color: Colors.grey),
          )),
        // Column(
        //   children: [
        //     Text("original video Info"),
        //     Text(fileMedia.path),
        //     Text("Size ${videoSize / 1000} KB")
        //   ],
        // ),
        // if (compressVideoInfo == null)
        //   Container(child: CircularProgressIndicator()),
        if (compressVideoInfo != null && secondsFlag == false)
          Container(child: builVideoCompressedInfo()),
        SizedBox(
          height: 24,
        ),
      ],
    );
  }

  _buildShareButtomBar() {
    return Positioned(
      bottom: MediaQuery.of(context).viewInsets.bottom,
      left: 0,
      right: 0,
      child: Container(
        alignment: Alignment.bottomCenter,
        padding: EdgeInsets.all(ScreenUtil().setWidth(16)),
        decoration: BoxDecoration(
          color: Colors.white,
          boxShadow: <BoxShadow>[
            BoxShadow(
              color: Colors.grey[300]!,
              offset: new Offset(5.0, 5.0),
              blurRadius: 20.0,
            )
          ],
        ),
        child: Row(
          children: [
            Expanded(flex: 2, child: _buildHashtag()),
            Expanded(flex: 5, child: _buildAttachment()),
            Flexible(flex: 1, child: _buildShareButton(this.widget.currentUser))
          ],
        ),
      ),
    );
  }

  _buildHashtag() {
    return Container(
      alignment: Alignment.bottomLeft,
      child: TextButton(
        onPressed: () {
          descriptionController.text += ("#");
          descriptionController.selection = TextSelection.fromPosition(
              TextPosition(offset: descriptionController.text.length));
        },
        child: Text(
          this.widget.loc.translate("#Add hashtag"),
          style: TextStyle(color: Colors.black),
        ),
      ),
    );
  }

  _buildAttachment() {
    return Container(
      child: Column(
        children: [
          Container(
            child: Row(
              children: [
                myType == null || isPoll == true
                    ? Expanded(
                        child: IconButton(
                          icon: Icon(
                            Icons.poll,
                          ),
                          onPressed: () {
                            setState(() {
                              isPollForm = true;
                            });
                          },
                        ),
                      )
                    : Expanded(
                        child: IconButton(
                          icon: Icon(
                            Icons.poll,
                            color: Colors.grey,
                          ),
                          onPressed: null,
                        ),
                      ),
                myType == "Myvideo" || myType == null && isPoll == false
                    ? Expanded(
                        child: IconButton(
                          icon: const Icon(
                            Icons.videocam_outlined,
                          ),
                          onPressed: () => capture(MediaSource.video),
                        ),
                      )
                    : Expanded(
                        child: IconButton(
                          icon: const Icon(
                            Icons.videocam_outlined,
                            color: Colors.grey,
                          ),
                          onPressed: () => null,
                        ),
                      ),
                myType == "Myimg" || myType == null && isPoll == false
                    ? Expanded(
                        child: Container(
                          child: IconButton(
                              icon: Icon(Icons.camera_alt_outlined),
                              onPressed: loadAssets),
                        ),
                      )
                    : Expanded(
                        child: Container(
                          child: IconButton(
                              icon: Icon(
                                Icons.camera_alt_outlined,
                                color: Colors.grey,
                              ),
                              onPressed: null),
                        ),
                      ),
                myType == "Mygif" || myType == null && isPoll == false
                    ? Expanded(
                        child: Container(
                          child: IconButton(
                            icon: Icon(
                              Icons.gif,
                            ),
                            onPressed: () async {
                              focusNodeTitle!.unfocus();
                              focusNodeDescription!.unfocus();

                              GiphyGif? res = await GiphyPicker.pickGif(
                                context: context,
                                fullScreenDialog: false,
                                showPreviewPage: true,
                                apiKey: 'EwNGOsJ61NlXaGQ3Nw6ZwxYjZNTyRoW2',
                                decorator: GiphyDecorator(
                                  showAppBar: false,
                                  searchElevation: 4,
                                  giphyTheme: ThemeData.dark(),
                                ),
                              );
                              setState(() {
                                res == null ? myType = null : myType = "Mygif";
                                _gif = res;
                              });
                            },
                          ),
                        ),
                      )
                    : Expanded(
                        child: Container(
                          child: IconButton(
                            icon: Icon(
                              Icons.gif,
                              color: Colors.grey,
                            ),
                            onPressed: null,
                          ),
                        ),
                      ),
              ],
            ),
          ),
        ],
      ),
    );
  }

  _buildShareButton(UserModel? currentUser) {
    if (titleController.text.isEmpty ||
        descriptionController.text.isEmpty ||
        secondsFlag == true ||
        (fileMedia != null && compressVideoInfo == null)) {
      validColor = false;
    } else {
      validColor = true;
    }
    return Container(
        child: ClipOval(
            child: Material(
                color: validColor == true ? secondColor : Colors.grey[400],
                child: IconButton(
                  icon: Icon(Icons.send_rounded),
                  color: Colors.white,
                  iconSize: 30,
                  onPressed: validColor == false
                      ? () => {}
                      : () => validationPost(currentUser),
                ))));
  }

  _buildHashtagField() {
    return TextFormField(
      textDirection: checkIsHebrew(descriptionController.text)
          ? TextDirection.rtl
          : TextDirection.ltr,
      onChanged: (text) {
        if (titleController.text.isEmpty ||
            descriptionController.text.isEmpty &&
                (fileMedia != null && compressVideoInfo == null)) {
          setState(() {
            validColor = false;
          });
        } else {
          setState(() {
            validColor = true;
          });
        }
      },
      focusNode: focusNodeDescription,
      keyboardType: TextInputType.multiline,
      maxLines: null,
      decoration: InputDecoration.collapsed(
          hintText: this
              .widget
              .loc
              .translate("Please type all you want to talk about...")),
      minLines: 6,
      controller: descriptionController,
    );
  }

  builVideoCompressedInfo() {
    if (compressVideoInfo == null) return Container();
    return Container(
        child: Text(
      this.widget.loc.translate("The file is received in the system!"),
      style: TextStyle(color: Colors.grey),
    ));
  }
}

class VideoCompressApi {
  static Future<MediaInfo?> compressVideo(File file) async {
    try {
      await VideoCompress.setLogLevel(0);
      return VideoCompress.compressVideo(
        file.path,
        quality: VideoQuality.LowQuality,
        includeAudio: true,
      );
    } catch (e) {
      VideoCompress.cancelCompression();
    }
  }
}
